@@ -11,6 +11,7 @@ pub enum FormatCode {
     Config,
     ScriptName,
     Regex,
+    RangeQuery,
     ScriptQuery,
     Tag,
     FilterQuery,
@@ -37,13 +38,18 @@ pub enum Error {
     NoAlias(String),
     UnknownType(String),
     Format(FormatCode, String),
+
     ScriptError(i32),
     PreRunError(i32),
+    EditorError(i32, Vec<String>),
+
     RedundantOpt(RedundantOpt),
     TagFilterNotFound(String),
     NoPreviousArgs,
     DontFuzz,
     Empty,
+
+    Completion,
 }
 
 impl<T: 'static + Send + Sync + std::error::Error> From<T> for Error {
@@ -91,6 +97,7 @@ impl<T, E: 'static + Send + Sync + std::error::Error> Contextable<T> for std::re
 }
 #[derive(Debug, Clone)]
 pub enum RedundantOpt {
+    Scripts(Vec<String>),
     CommandArgs,
     Type,
     Tag,
