@@ -40,6 +40,7 @@ impl Display for Error {
                 write!(f, "Format error for ")?;
                 use FormatCode::*;
                 match code {
+                    RangeQuery => write!(f, "range query")?,
                     Config => write!(f, "config file")?,
                     ScriptName => write!(f, "script name")?,
                     Regex => write!(f, "regular expression")?,
@@ -54,6 +55,11 @@ impl Display for Error {
                 write!(f, " '{}'", s)?;
             }
             ScriptError(code) => write!(f, "Script exited unexpectedly with {}", code)?,
+            PreRunError(code) => write!(f, "Pre-dun script exited unexpectedly with {}", code)?,
+            EditorError(code, cmd) => {
+                let cmd = cmd.join(" ");
+                write!(f, "Editor `{}` exited unexpectedly with {}", cmd, code)?
+            }
             NoAlias(alias) => write!(f, "No such alias: {}", alias)?,
             RedundantOpt(opt) => write!(f, "Redundant option: {:?}", opt)?,
             _ => {
