@@ -4,6 +4,7 @@ use crate::error::{Error, Result};
 use crate::fuzzy;
 use crate::script::{IntoScriptName, ScriptInfo};
 use crate::script_repo::{RepoEntry, ScriptRepo};
+use crate::util::get_display_type;
 use fxhash::FxHashSet as HashSet;
 use std::sync::Once;
 
@@ -32,12 +33,17 @@ pub async fn do_list_query<'a>(
         // SAFETY: `mem` 已保證回傳的陣列不可能包含相同的資料
         let repo = unsafe { &mut *repo_ptr };
         match query {
-            ListQuery::Pattern(re) => {
+            ListQuery::Pattern(re, og) => {
+                let mut is_empty = true;
                 for script in repo.iter_mut(false) {
                     if re.is_match(&script.name.key()) {
+                        is_empty = false;
                         insert!(script);
                     }
                 }
+                if is_empty {
+                    return Err(Error::ScriptNotFound(og.to_owned()));
+                }
             }
             ListQuery::Query(query) => {
                 let script = match do_script_query_strict(query, repo).await {
@@ -49,7 +55,12 @@ pub async fn do_list_query<'a>(
             }
         }
     }
-    Ok(ret)
+    if ret.is_empty() {
+        log::debug!("列表查不到東西，卻又不是因為 pattern not match，想必是因為使用者取消了模糊搜");
+        Err(Error::DontFuzz)
+    } else {
+        Ok(ret)
+    }
 }
 
 pub async fn do_script_query<'b>(
@@ -146,11 +157,11 @@ fn prompt_fuzz_acceptable(script: &ScriptInfo) -> Result {
 
     let term = Term::stderr();
 
-    let color = Config::get().get_color(&script.ty)?;
+    let ty = get_display_type(&script.ty);
     let msg = format!(
         "{} [Y/N]",
-        format!("{}({})?", script.name, script.ty)
-            .color(color)
+        format!("{}({})?", script.name, ty.display())
+            .color(ty.color())
             .bold(),
     );
 
