@@ -3,6 +3,7 @@ use crate::error::{Contextable, Error, FormatCode::ScriptName as ScriptNameCode,
 use crate::script_time::ScriptTime;
 use crate::script_type::ScriptType;
 use crate::tag::Tag;
+use crate::util::illegal_name;
 use chrono::NaiveDateTime;
 use fxhash::FxHashSet as HashSet;
 use serde::{Deserialize, Serialize};
@@ -38,7 +39,7 @@ impl FromStr for ScriptName {
 impl ScriptName {
     pub fn valid(s: &str) -> Result<Option<u32>> {
         log::debug!("檢查腳本名：{}", s);
-        let reg = regex::Regex::new(r"^\.(\w+)$")?;
+        let reg = regex::Regex::new(r"^\.(\d+)$")?;
         let m = reg.captures(s);
         if let Some(m) = m {
             let id_str = m.get(1).unwrap().as_str();
@@ -48,14 +49,10 @@ impl ScriptName {
             }
         } else {
             // FIXME: 好好想想什麼樣的腳本名可行，並補上單元測試
-            if s.starts_with('-')
-                || s.starts_with('.')
-                || s.contains("..")
-                || s.contains(' ')
-                || s.is_empty()
-            {
-                return Err(Error::Format(ScriptNameCode, s.to_owned()))
-                    .context("命名腳本格式有誤");
+            for s in s.split('/') {
+                if illegal_name(s) {
+                    return Err(Error::Format(ScriptNameCode, s.to_owned()));
+                }
             }
             Ok(None)
         }
@@ -81,8 +78,28 @@ impl ScriptName {
     }
     /// 回傳值是相對於 HS_HOME 的路徑
     pub fn to_file_path(&self, ty: &ScriptType) -> Result<PathBuf> {
-        fn add_ext(name: &mut String, ty: &ScriptType) -> Result<()> {
-            if let Some(ext) = &Config::get().get_script_conf(ty)?.ext {
+        self.to_file_path_inner(ty, false)
+    }
+    /// 回傳值是相對於 HS_HOME 的路徑，對未知的類別直接用類別名作擴展名
+    pub fn to_file_path_fallback(&self, ty: &ScriptType) -> PathBuf {
+        self.to_file_path_inner(ty, true).unwrap()
+    }
+    fn to_file_path_inner(&self, ty: &ScriptType, fallback: bool) -> Result<PathBuf> {
+        fn add_ext(name: &mut String, ty: &ScriptType, fallback: bool) -> Result {
+            let ext = match Config::get().get_script_conf(ty) {
+                Err(e) => {
+                    if !fallback {
+                        return Err(e);
+                    }
+                    log::warn!(
+                        "取腳本路徑時找不到類別設定：{}，直接把類別名當擴展名試試",
+                        e,
+                    );
+                    Some(ty.as_ref())
+                }
+                Ok(c) => c.ext.as_ref().map(|s| s.as_ref()),
+            };
+            if let Some(ext) = ext {
                 *name = format!("{}.{}", name, ext);
             }
             Ok(())
@@ -91,12 +108,12 @@ impl ScriptName {
             ScriptName::Anonymous(id) => {
                 let mut file_name = id.to_string();
                 let dir: PathBuf = ANONYMOUS.into();
-                add_ext(&mut file_name, ty)?;
+                add_ext(&mut file_name, ty, fallback)?;
                 Ok(dir.join(file_name))
             }
             ScriptName::Named(name) => {
                 let mut file_name = name.to_string();
-                add_ext(&mut file_name, ty)?;
+                add_ext(&mut file_name, ty, fallback)?;
                 Ok(file_name.into())
             }
         }
@@ -136,6 +153,7 @@ pub struct TimelessScriptInfo {
 pub struct ScriptInfo {
     pub read_time: ScriptTime,
     pub write_time: ScriptTime,
+    pub neglect_time: Option<ScriptTime>,
     /// (content, args)
     pub exec_time: Option<ScriptTime<(String, String)>>,
     pub exec_done_time: Option<ScriptTime<i32>>,
@@ -174,6 +192,7 @@ impl ScriptInfo {
             write_time: now.clone(),
             exec_time: None,
             exec_done_time: None,
+            neglect_time: None,
             timeless_info,
         }
     }
@@ -192,8 +211,8 @@ impl ScriptInfo {
             map(&self.exec_done_time)
         )
     }
-    pub fn file_path(&self) -> Result<PathBuf> {
-        self.name.to_file_path(&self.ty)
+    pub fn file_path_fallback(&self) -> PathBuf {
+        self.name.to_file_path_fallback(&self.ty)
     }
     pub fn read(&mut self) {
         self.read_time = ScriptTime::now(());
@@ -213,6 +232,9 @@ impl ScriptInfo {
         log::trace!("{:?} 執行結果為 {}", self, code);
         self.exec_done_time = Some(ScriptTime::now(code));
     }
+    pub fn neglect(&mut self) {
+        self.neglect_time = Some(ScriptTime::now(()))
+    }
     pub fn builder(
         id: i64,
         name: ScriptName,
@@ -229,6 +251,7 @@ impl ScriptInfo {
             exec_time: None,
             write_time: None,
             exec_done_time: None,
+            neglect_time: None,
         }
     }
 }
@@ -260,6 +283,7 @@ pub struct ScriptBuilder {
     created_time: Option<NaiveDateTime>,
     write_time: Option<NaiveDateTime>,
     exec_time: Option<NaiveDateTime>,
+    neglect_time: Option<NaiveDateTime>,
     exec_done_time: Option<NaiveDateTime>,
     id: i64,
     tags: HashSet<Tag>,
@@ -283,6 +307,10 @@ impl ScriptBuilder {
         self.write_time = Some(time);
         self
     }
+    pub fn neglect_time(mut self, time: NaiveDateTime) -> Self {
+        self.neglect_time = Some(time);
+        self
+    }
     pub fn created_time(mut self, time: NaiveDateTime) -> Self {
         self.created_time = Some(time);
         self
@@ -294,6 +322,7 @@ impl ScriptBuilder {
             read_time: ScriptTime::new_or(self.read_time, created_time.clone()),
             exec_time: self.exec_time.map(ScriptTime::new),
             exec_done_time: self.exec_done_time.map(ScriptTime::new),
+            neglect_time: self.neglect_time.map(ScriptTime::new),
             timeless_info: TimelessScriptInfo {
                 changed: false,
                 id: self.id,
