@@ -23,6 +23,7 @@ pub struct DBEnv {
     no_trace: bool,
     info_pool: SqlitePool,
     historian: Historian,
+    modifies_script: bool,
 }
 
 pub struct RepoEntryOptional<'b> {
@@ -41,27 +42,8 @@ impl<'b> RepoEntryOptional<'b> {
         let info = self.entry.or_insert(info);
         if !exist {
             log::debug!("往資料庫塞新腳本 {:?}", info);
-            let name_cow = info.name.key();
-            let name = name_cow.as_ref();
-            let ty = info.ty.as_ref();
-            let tags = join_tags(info.tags.iter());
-            sqlx::query!(
-                "
-                INSERT INTO script_infos (name, ty, tags)
-                VALUES(?, ?, ?)
-                ",
-                name,
-                ty,
-                tags,
-            )
-            .execute(&self.env.info_pool)
-            .await?;
-            log::debug!("往資料庫新增腳本成功");
-            let id = sqlx::query!("SELECT last_insert_rowid() as id")
-                .fetch_one(&self.env.info_pool)
-                .await?
-                .id;
-            log::debug!("得到新腳本 id {}", id);
+            let id = self.env.handle_insert(info).await?;
+            log::debug!("往資料庫新增腳本成功，得 id = {}", id);
             info.set_id(id as i64);
         }
         Ok(RepoEntry::new(info, self.env))
@@ -69,11 +51,15 @@ impl<'b> RepoEntryOptional<'b> {
 }
 
 impl DBEnv {
-    pub async fn purge_last_events(&self, id: i64) -> Result {
-        log::debug!("清理腳本 {:?} 的最新事件", id);
-        sqlx::query!("DELETE FROM last_events WHERE script_id = ?", id)
-            .execute(&self.info_pool)
-            .await?;
+    pub async fn handle_neglect(&self, id: i64) -> Result {
+        let time = Utc::now().naive_utc();
+        sqlx::query!(
+            "UPDATE last_events SET neglect = ? WHERE script_id = ?",
+            time,
+            id
+        )
+        .execute(&self.info_pool)
+        .await?;
         Ok(())
     }
 
@@ -81,22 +67,66 @@ impl DBEnv {
         let last_time = info.last_time();
         let exec_time = info.exec_time.as_ref().map(|t| **t);
         let exec_done_time = info.exec_done_time.as_ref().map(|t| **t);
+        let neglect = info.neglect_time.as_ref().map(|t| **t);
         sqlx::query!(
-            "INSERT OR REPLACE INTO last_events (script_id, last_time, read, write, exec, exec_done) VALUES(?, ?, ?, ?, ?, ?)",
+            "
+            INSERT OR REPLACE INTO last_events
+            (script_id, last_time, read, write, exec, exec_done, neglect) VALUES(?, ?, ?, ?, ?, ?, ?)
+            ",
             info.id,
             last_time,
             *info.read_time,
             *info.write_time,
             exec_time,
             exec_done_time,
+            neglect
         )
         .execute(&self.info_pool)
         .await?;
         Ok(())
     }
+
+    async fn handle_delete(&self, id: i64) -> Result {
+        assert!(self.modifies_script);
+        self.historian.remove(id).await?;
+        log::debug!("清理腳本 {:?} 的最新事件", id);
+        sqlx::query!("DELETE FROM last_events WHERE script_id = ?", id)
+            .execute(&self.info_pool)
+            .await?;
+        sqlx::query!("DELETE from script_infos where id = ?", id)
+            .execute(&self.info_pool)
+            .await?;
+        Ok(())
+    }
+
+    async fn handle_insert(&self, info: &ScriptInfo) -> Result<i64> {
+        assert!(self.modifies_script);
+        let name_cow = info.name.key();
+        let name = name_cow.as_ref();
+        let ty = info.ty.as_ref();
+        let tags = join_tags(info.tags.iter());
+        sqlx::query!(
+            "
+            INSERT INTO script_infos (name, ty, tags)
+            VALUES(?, ?, ?)
+            ",
+            name,
+            ty,
+            tags,
+        )
+        .execute(&self.info_pool)
+        .await?;
+        let id = sqlx::query!("SELECT last_insert_rowid() as id")
+            .fetch_one(&self.info_pool)
+            .await?
+            .id;
+        Ok(id as i64)
+    }
+
     async fn handle_change(&self, info: &ScriptInfo, main_event_id: i64) -> Result<i64> {
         log::debug!("開始修改資料庫 {:?}", info);
         if info.changed {
+            assert!(self.modifies_script);
             let name = info.name.key();
             let name = name.as_ref();
             let tags = join_tags(info.tags.iter());
@@ -199,10 +229,6 @@ pub struct ScriptRepo {
 }
 
 impl ScriptRepo {
-    pub fn get_db_env(&self) -> &DBEnv {
-        &self.db_env
-    }
-
     pub fn iter_known_tags(&self) -> impl Iterator<Item = &Tag> {
         self.known_tags.iter()
     }
@@ -235,6 +261,7 @@ impl ScriptRepo {
         recent: Option<RecentFilter>,
         historian: Historian,
         no_trace: bool,
+        modifies_script: bool,
     ) -> Result<ScriptRepo> {
         let mut known_tags: HashSet<Tag> = Default::default();
 
@@ -251,16 +278,16 @@ impl ScriptRepo {
         .fetch_all(&pool)
         .await?;
         let mut map: HashMap<String, ScriptInfo> = Default::default();
-        for script in scripts.into_iter() {
-            let name = script.name;
-            log::trace!("載入腳本：{} {} {}", name, script.ty, script.tags);
+        for record in scripts.into_iter() {
+            let name = record.name;
+            log::trace!("載入腳本：{} {} {}", name, record.ty, record.tags);
             let script_name = name.clone().into_script_name()?;
 
             let mut builder = ScriptInfo::builder(
-                script.id,
+                record.id,
                 script_name,
-                script.ty.into(),
-                script.tags.split(',').filter_map(|s| {
+                record.ty.into(),
+                record.tags.split(',').filter_map(|s| {
                     if s.is_empty() {
                         None
                     } else {
@@ -273,24 +300,31 @@ impl ScriptRepo {
                     }
                 }),
             )
-            .created_time(script.created_time);
+            .created_time(record.created_time);
 
-            if let Some(time) = script.write {
+            if let Some(time) = record.write {
                 builder = builder.write_time(time);
             }
-            if let Some(time) = script.read {
+            if let Some(time) = record.read {
                 builder = builder.read_time(time);
             }
-            if let Some(time) = script.exec {
+            if let Some(time) = record.exec {
                 builder = builder.exec_time(time);
             }
-            if let Some(time) = script.exec_done {
+            if let Some(time) = record.exec_done {
                 builder = builder.exec_done_time(time);
             }
+            if let Some(time) = record.neglect {
+                builder = builder.neglect_time(time);
+            }
 
             let script = builder.build();
 
-            let hide_by_time = if let Some((time_bound, archaeology)) = time_bound {
+            let hide_by_time = if let Some((mut time_bound, archaeology)) = time_bound {
+                if let Some(neglect) = record.neglect {
+                    log::debug!("腳本 {} 曾於 {} 被忽略", script.name, neglect);
+                    time_bound = std::cmp::max(neglect, time_bound);
+                }
                 let overtime = time_bound > script.last_time_without_read();
                 archaeology ^ overtime
             } else {
@@ -311,6 +345,7 @@ impl ScriptRepo {
                 info_pool: pool,
                 no_trace,
                 historian,
+                modifies_script,
             },
         })
     }
@@ -366,15 +401,15 @@ impl ScriptRepo {
             .get_mut(&*name.key())
             .map(|info| RepoEntry::new(info, db_env))
     }
+    pub fn get_mut_by_id(&mut self, id: i64) -> Option<RepoEntry<'_>> {
+        // XXX: 複雜度很瞎
+        self.iter_mut(true).find(|e| e.id == id)
+    }
 
     pub async fn remove(&mut self, name: &ScriptName) -> Result {
         if let Some(info) = self.map.remove(&*name.key()) {
             log::debug!("從資料庫刪除腳本 {:?}", info);
-            self.db_env.historian.remove(info.id).await?;
-            self.db_env.purge_last_events(info.id).await?;
-            sqlx::query!("DELETE from script_infos where id = ?", info.id)
-                .execute(&self.db_env.info_pool)
-                .await?;
+            self.db_env.handle_delete(info.id).await?;
         }
         Ok(())
     }
