@@ -1,3 +1,5 @@
+use crate::args::{AliasRoot, Completion, List, Root, Subs};
+use crate::config::Config;
 use crate::error::{Contextable, Error, RedundantOpt, Result};
 use crate::path;
 use crate::query::{self, EditQuery};
@@ -5,6 +7,7 @@ use crate::script::{IntoScriptName, ScriptInfo, ScriptName};
 use crate::script_repo::{RepoEntry, ScriptRepo};
 use crate::script_type::{iter_default_templates, ScriptType};
 use crate::tag::{Tag, TagFilter};
+use crate::Either;
 use hyper_scripter_historian::Historian;
 use std::path::{Path, PathBuf};
 
@@ -191,6 +194,7 @@ pub async fn run_n_times(
             &*entry,
             &args,
             &entry.exec_time.as_ref().unwrap().data().unwrap().0,
+            entry.last_event_id(),
         );
         let ret_code: i32;
         match run_res {
@@ -252,6 +256,19 @@ pub fn load_templates() -> Result {
     Ok(())
 }
 
+/// 判斷是否需要寫入主資料庫（script_infos 表格）
+pub fn need_write(arg: &Subs) -> bool {
+    use Subs::*;
+    match arg {
+        Edit { .. } => true,
+        MV { .. } => true,
+        CP { .. } => true,
+        RM { .. } => true,
+        LoadUtils { .. } => true,
+        _ => false,
+    }
+}
+
 use super::PrepareRespond;
 pub async fn after_script(
     entry: &mut RepoEntry<'_>,
@@ -271,6 +288,7 @@ pub async fn after_script(
                     false
                 } else {
                     log::info!("舊本未變動，不記錄事件");
+                    // FIXME: 其實應該記錄一筆讀取事件…
                     true
                 });
             }
@@ -279,3 +297,57 @@ pub async fn after_script(
     entry.update(|info| info.write()).await?;
     Ok(true)
 }
+
+pub fn handle_completion(root: Either<Root, Completion>) -> Result<Root> {
+    use structopt::StructOpt;
+    match root {
+        Either::One(root) => Ok(root),
+        Either::Two(completion) => match completion {
+            Completion::LS { args } => {
+                let mut new_root = match Root::from_iter_safe(args) {
+                    Ok(t) => t,
+                    Err(e) => {
+                        log::warn!("補全時出錯 {}", e);
+                        // NOTE: -V 或 --help 也會走到這裡
+                        return Err(Error::Completion);
+                    }
+                };
+                new_root.home_is_set = false;
+                new_root.dump_args = false;
+                new_root.subcmd = Some(Subs::LS(List {
+                    name: true,
+                    plain: true,
+                    ..Default::default()
+                }));
+                new_root.sanitize_flags();
+                log::info!("補完模式，參數為 {:?}", new_root);
+                Ok(new_root)
+            }
+            Completion::Alias { args } => {
+                match AliasRoot::from_iter_safe(&args) {
+                    Ok(alias_root) => {
+                        fn print_iter<T: std::fmt::Display>(iter: impl Iterator<Item = T>) {
+                            for arg in iter {
+                                print!("{} ", arg);
+                            }
+                        }
+
+                        let p = path::compute_home_path_optional(alias_root.hs_home.as_ref())?;
+                        let conf = Config::load(&p)?;
+                        if let Some(new_args) = alias_root.expand_alias(&args, &conf) {
+                            print_iter(new_args);
+                        } else {
+                            print_iter(args.iter());
+                        };
+                        std::process::exit(0);
+                    }
+                    Err(e) => {
+                        log::warn!("展開別名時出錯 {}", e);
+                        // NOTE: -V 或 --help 也會走到這裡
+                        Err(Error::Completion)
+                    }
+                }
+            }
+        },
+    }
+}
