@@ -1,9 +1,12 @@
 use crate::config::Config;
 use crate::error::{Contextable, Error, Result};
+use crate::extract_msg::extract_env_from_content;
 use crate::path::{get_home, get_template_path};
 use crate::script::ScriptInfo;
 use crate::script_type::{get_default_template, ScriptType};
 use chrono::{DateTime, Utc};
+use colored::Color;
+use std::borrow::Cow;
 use std::ffi::OsStr;
 use std::fs::{create_dir_all, remove_file, rename, File};
 use std::io::{Read, Write};
@@ -12,12 +15,22 @@ use std::process::{Command, ExitStatus};
 
 pub mod main_util;
 
+pub fn illegal_name(s: &str) -> bool {
+    s.starts_with('-')
+        || s.starts_with('.')
+        || s.contains("..")
+        || s.contains(' ')
+        || s.contains('/')
+        || s.is_empty()
+}
+
 // XXX: main util?
 pub fn run(
     script_path: &Path,
     info: &ScriptInfo,
     remaining: &[String],
     content: &str,
+    run_id: i64,
 ) -> Result<()> {
     let conf = Config::get();
     let ty = &info.ty;
@@ -30,6 +43,8 @@ pub fn run(
 
     let hs_cmd = std::env::args().next().unwrap_or_default();
 
+    let hs_env_help: Vec<_> = extract_env_from_content(content).collect();
+
     let script_conf = conf.get_script_conf(ty)?;
     let cmd_str = if let Some(cmd) = &script_conf.cmd {
         cmd
@@ -47,9 +62,11 @@ pub fn run(
     info = json!({
         "path": script_path,
         "hs_home": hs_home,
+        "hs_run_id": run_id,
         "hs_tags": hs_tags,
         "hs_cmd": hs_cmd,
         "hs_exe": hs_exe,
+        "hs_env_help": hs_env_help,
         "name": name,
         "content": content,
     });
@@ -162,19 +179,6 @@ pub fn write_file(path: &Path, content: &str) -> Result<()> {
 pub fn remove(script_path: &Path) -> Result<()> {
     handle_fs_res(&[&script_path], remove_file(&script_path))
 }
-pub fn change_name_only<F: Fn(&str) -> String>(full_name: &str, transform: F) -> String {
-    let mut arr: Vec<_> = full_name.split('/').collect();
-    let len = arr.len();
-    if len == 0 {
-        unreachable!();
-    } else if len == 1 {
-        transform(full_name)
-    } else {
-        let new = transform(arr[len - 1]);
-        arr[len - 1] = &new;
-        arr.join("/")
-    }
-}
 pub fn mv(origin: &Path, new: &Path) -> Result<()> {
     log::info!("修改 {:?} 為 {:?}", origin, new);
     // NOTE: 創建資料夾和檔案
@@ -240,13 +244,6 @@ pub fn to_display_args(arg: String) -> Result<String> {
     }
 }
 
-pub fn serialize_to_string<S: serde::Serializer, T: ToString>(
-    t: T,
-    serializer: S,
-) -> std::result::Result<S::Ok, S::Error> {
-    serializer.serialize_str(&t.to_string())
-}
-
 fn relative_to_home(p: &Path) -> Option<&Path> {
     const CUR_DIR: &str = ".";
     let home = dirs::home_dir()?;
@@ -346,3 +343,33 @@ fn write_prepare_script<W: Write>(
             e => panic!("解析模版錯誤：{}", e),
         })
 }
+
+pub struct DisplayType<'a> {
+    ty: &'a ScriptType,
+    color: Option<Color>,
+}
+impl<'a> DisplayType<'a> {
+    pub fn is_unknown(&self) -> bool {
+        self.color.is_none()
+    }
+    pub fn color(&self) -> Color {
+        self.color.unwrap_or(Color::BrightBlack)
+    }
+    pub fn display(&self) -> Cow<'a, str> {
+        if self.is_unknown() {
+            Cow::Owned(format!("{}, unknown", self.ty))
+        } else {
+            Cow::Borrowed(self.ty.as_ref())
+        }
+    }
+}
+pub fn get_display_type(ty: &ScriptType) -> DisplayType {
+    let conf = Config::get();
+    match conf.get_color(ty) {
+        Err(e) => {
+            log::warn!("取腳本顏色時出錯：{}，視為未知類別", e);
+            DisplayType { ty, color: None }
+        }
+        Ok(c) => DisplayType { ty, color: Some(c) },
+    }
+}
