@@ -1,9 +1,8 @@
-use hyper_scripter::config::{Config, PromptLevel, RawConfig};
+use hyper_scripter::config::{Config, PromptLevel};
 use std::fs::canonicalize;
 use std::io::{BufRead, BufReader};
 use std::path::PathBuf;
-use std::process::ExitStatus;
-use std::process::{Command, Stdio};
+use std::process::{Command, ExitStatus, Stdio};
 use std::sync::{Mutex, MutexGuard, Once};
 
 lazy_static::lazy_static! {
@@ -24,11 +23,41 @@ pub fn get_exe_abs() -> String {
 
 const HOME: &str = "./.hyper_scripter";
 
+#[derive(Debug)]
+enum ErrorInner {
+    Other,
+    ExitStatus(ExitStatus),
+}
+#[derive(Debug)]
+pub struct Error {
+    msg: Vec<String>,
+    inner: ErrorInner,
+}
+impl Error {
+    pub fn exit_status(e: ExitStatus) -> Error {
+        Error {
+            msg: vec![],
+            inner: ErrorInner::ExitStatus(e),
+        }
+    }
+    pub fn other<T: ToString>(s: T) -> Error {
+        Error {
+            msg: vec![s.to_string()],
+            inner: ErrorInner::Other,
+        }
+    }
+    pub fn context<T: ToString>(mut self, s: T) -> Error {
+        self.msg.push(s.to_string());
+        self
+    }
+}
+type Result<T = ()> = std::result::Result<T, Error>;
+
 pub fn get_home() -> PathBuf {
     canonicalize(HOME).unwrap()
 }
 pub fn load_conf() -> Config {
-    RawConfig::load().unwrap().unwrap().0.into()
+    Config::load(hyper_scripter::path::get_home()).unwrap()
 }
 pub fn setup<'a>() -> MutexGuard<'a, ()> {
     let g = setup_with_utils();
@@ -50,7 +79,7 @@ pub fn setup_with_utils<'a>() -> MutexGuard<'a, ()> {
 
     static ONCE: Once = Once::new();
     ONCE.call_once(|| {
-        hyper_scripter::path::set_home(&home).unwrap();
+        hyper_scripter::path::set_home(Some(&home)).unwrap();
         Config::init().unwrap();
     });
 
@@ -79,11 +108,11 @@ pub fn check_exist(p: &[&str]) -> bool {
     let file = join_path(p);
     file.exists()
 }
-pub fn run<T: ToString>(args: T) -> Result<String, ExitStatus> {
+pub fn run<T: ToString>(args: T) -> Result<String> {
     let home = get_home();
     run_with_home(&*home.to_string_lossy(), args)
 }
-pub fn run_with_home<T: ToString>(home: &str, args: T) -> Result<String, ExitStatus> {
+pub fn run_with_home<T: ToString>(home: &str, args: T) -> Result<String> {
     let mut full_args = vec!["-H", home];
     let args = args.to_string();
     let args_vec: Vec<&str> = if args.find('|').is_some() {
@@ -119,7 +148,7 @@ pub fn run_with_home<T: ToString>(home: &str, args: T) -> Result<String, ExitSta
     let res = if status.success() {
         Ok(out_str.join("\n"))
     } else {
-        Err(status)
+        Err(Error::exit_status(status).context(format!("執行 {:?} 失敗", args_vec)))
     };
     log::info!("執行 {:?} 完畢，結果為 {:?}", args_vec, res);
     res
@@ -159,6 +188,9 @@ pub struct ScriptTest {
     name: String,
 }
 impl ScriptTest {
+    pub fn get_name(&self) -> &str {
+        &self.name
+    }
     pub fn new(name: &str, tags: Option<&str>) -> Self {
         let tags_str = tags.map(|s| format!("-t {}", s)).unwrap_or_default();
         run(format!("e {} ={} | echo $HS_TAGS", tags_str, name)).unwrap();
@@ -167,10 +199,25 @@ impl ScriptTest {
         }
     }
     pub fn assert_not_exist(&self, args: Option<&str>, msg: Option<&str>) {
-        let s = format!("which {} ={}", args.unwrap_or_default(), self.name);
+        let s = format!("cat {} ={}", args.unwrap_or_default(), self.name);
         let msg = msg.map(|s| format!("\n{}", s)).unwrap_or_default();
         run(&s).expect_err(&format!("{} 找到東西{}", s, msg));
     }
+    pub fn archaeology<'a>(&'a self) -> ScriptTestWithFilter<'a> {
+        ScriptTestWithFilter {
+            script: self,
+            filter: "-A",
+        }
+    }
+    pub fn filter<'a>(&'a self, filter: &'a str) -> ScriptTestWithFilter<'a> {
+        ScriptTestWithFilter {
+            script: self,
+            filter,
+        }
+    }
+    pub fn run(&self, args: &str) -> Result<String> {
+        self.filter("").run(args)
+    }
     pub fn assert_tags<const N: usize>(
         &self,
         tags: [&str; N],
@@ -191,4 +238,38 @@ impl ScriptTest {
             self.name, msg
         );
     }
+    pub fn can_find(&self, command: &str) -> Result {
+        self.filter("").can_find(command)
+    }
+    pub fn can_find_by_name(&self) -> Result {
+        self.filter("").can_find_by_name()
+    }
+}
+pub struct ScriptTestWithFilter<'a> {
+    script: &'a ScriptTest,
+    filter: &'a str,
+}
+impl<'a> ScriptTestWithFilter<'a> {
+    pub fn run(&self, args: &str) -> Result<String> {
+        let s = format!("{} ={} {}", self.filter, self.script.name, args);
+        run(&s)
+    }
+    pub fn can_find(&self, command: &str) -> Result {
+        let command = format!(
+            "{} ls --plain --grouping=none --name {}",
+            self.filter, command
+        );
+        let res = run(&command)?;
+        if res == self.script.name {
+            Ok(())
+        } else {
+            Err(Error::other(format!(
+                "想找 {} 卻找到 {}",
+                self.script.name, res
+            )))
+        }
+    }
+    pub fn can_find_by_name(&self) -> Result {
+        self.can_find(&format!("={}", self.script.name))
+    }
 }
